package de.opalpiraten.scenepainter.sys;
import com.threerings.media.image.*;
import com.threerings.media.tile.*;
import com.threerings.media.tile.bundle.*;
import com.threerings.cast.bundle.*;
import com.threerings.cast.*;
import com.threerings.miso.tile.*;
import com.threerings.resource.*;
import java.io.*;
import java.net.*;
import java.util.*;


/**
 * The NenyaBrowser gives access to threerings.com nenya media library
 */
public class NenyaBrowser {

  /**
   * Resourcemanger to use for loading
   */
  private ResourceManager resmgr;
  
  /**
   * The Repository responsible for loading tiles
   */
  private BundledTileSetRepository btr;
  
  /**
   * The Repository for loading components
   */
  private BundledComponentRepository bcr;
  
  /**
   * Stores the objects required for recoloring (this is the default
   * ColorPository, which is used unless a project supplies its own).
   */
  private ColorPository cpos;
  
  /**
   * Responsible for serving the tiles
   */
  private MisoTileManager mtm;
  
  /**
   * Maps component resource identifiers to their numerical id.
   * The CRI is derived from concatening classname + "/" + componentname
   */
  private Hashtable compmap = new Hashtable();
  
  /**
   * Manages images
   */
  private ImageManager immgr;
   
  /**
   * Construct a new Nenyabrowser
   * @param nd config object, holding the required constants.
   * @param obs observer to be notified of loading progress (may be null)
   */
  public NenyaBrowser(NenyaDescriptor nd, ResourceManager.InitObserver obs) throws IOException {
    URLClassLoader ucl = new URLClassLoader(nd.jars);
    
    resmgr =new ResourceManager(nd.root,ucl);
    resmgr.initBundles(nd.rsrcdir,nd.manager,obs);
    cpos=ColorPository.loadColorPository(resmgr);
    immgr = new ImageManager(resmgr,new AWTImageCreator(null));
    if (nd.tilesets!=null) {
      btr =new BundledTileSetRepository(resmgr,immgr,nd.tilesets);
    }
    mtm = new MisoTileManager(resmgr,immgr);
    mtm.setTileSetRepository(btr);
    bcr = new BundledComponentRepository(resmgr,immgr,nd.components);
    
    try {
      Iterator it = bcr.enumerateComponentClasses();
      while (it.hasNext()) {
        ComponentClass cc = (ComponentClass)it.next();
        Iterator it2 = bcr.enumerateComponentIds(cc);
        while(it2.hasNext()) {
          Integer id = (Integer)it2.next();
          compmap.put(cc.name+"/"+bcr.getComponent(id.intValue()).name,id );
        }
      }
    }
    catch (Exception e) {
      e.printStackTrace();
    }
  }
  
  /**
   * Get the Manager responsible for serving tiles.
   */
  public TileManager getTileManager() {
    return mtm;
  }
  
  /**
   * Give access to the ImageManger
   */
  public ImageManager getImageManager() {
    return immgr;
  }
  
  /**
   * Query the names of all Tile resources supplied by this browser.
   * The names are sorted alphabetically
   */
  public String[] getTileNames() {
    String ret[] = new String[0];
    if (btr==null) return ret;
    
    try {
      Iterator it = btr.enumerateTileSets();
      Vector v = new Vector();
      while(it.hasNext()) {
        v.add(((TileSet)it.next()).getName());
      }
      ret = new String[v.size()];
      v.copyInto(ret);
      Arrays.sort(ret);
    }
    catch (Exception e) {
      e.printStackTrace();
    }
    return ret;
  }
  
  /**
   * Query the Component names.
   * @return a List of all component names prefixed by a slash and
   * their characterclass. This list is sorted alphabetically
   */
  public String[] getComponentNames() {
    Enumeration keys = compmap.keys();
    String[] ret = new String[compmap.size()];
    int i=0;
    while(keys.hasMoreElements()) {
      ret[i]=keys.nextElement().toString();
      i++;
    }
    Arrays.sort(ret);
    return ret;
  }
  
  /**
   * Maps a component resource identifier back to its id
   * @param cri classname + "/" +componentname
   */
  public int getComponentId(String cri) {
    Integer id;
    try {
      id = (Integer)compmap.get(cri);
    }
    catch (Exception e) { return -1;}
    return id.intValue();
  }
  
  /**
   * Where to get components from
   */
  public ComponentRepository getComponentRepository() {
    return bcr;
  }
  
  
  /**
   * Query the ColorPository storing the Objects for recoloring
   * @param n Name of the Project to get the Colorpository from.
   * if null, the default ColorPository is returned
   * @return either the ColorPository of the project or the default
   * Pository, if the Project has none itself
   */
  public ColorPository getColorPository() {
    return cpos;
  }
  
}

